# Copyright (c) 2026 Philipp Schafft

# licensed under Artistic License 2.0 (see LICENSE file)

# ABSTRACT: Perl module to provide generic, language independent, interfaces to language data


package Lingua::Generic::Interface;

use v5.20;
use strict;
use warnings;

use Carp;

our $VERSION = v0.01;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Lingua::Generic::Interface - Perl module to provide generic, language independent, interfaces to language data

=head1 VERSION

version v0.01

=head1 SYNOPSIS

    use Lingua::Generic::Interface;

=head1 AUTHOR

Philipp Schafft <lion@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2026 by Philipp Schafft <lion@cpan.org>.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut
